From 3ade8436b9c4a0c8d71a252068abf370ebe848ba Mon Sep 17 00:00:00 2001
From: Kien Nguyen <kien.n.quang@gmail.com>
Date: Wed, 24 Aug 2022 23:19:35 +0900
Subject: [PATCH] symlink: allow w32 non-admin user to create symlink

* src/w32.c (symlink): allow w32 user to create symlink without admin privileged

---
 src/w32.c | 19 +++++++++++++++++--
 1 file changed, 17 insertions(+), 2 deletions(-)

diff --git a/src/w32.c b/src/w32.c
index cbcfcdd4f..407befbda 100644
--- a/src/w32.c
+++ b/src/w32.c
@@ -5992,12 +5992,15 @@ sys_umask (int mode)
 #ifndef SYMBOLIC_LINK_FLAG_DIRECTORY
 #define SYMBOLIC_LINK_FLAG_DIRECTORY 0x1
 #endif
+#ifndef SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE
+#define SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE 0x2
+#endif
 
 int
 symlink (char const *filename, char const *linkname)
 {
   char linkfn[MAX_UTF8_PATH], *tgtfn;
-  DWORD flags = 0;
+  DWORD flags = SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE;
   int dir_access, filename_ends_in_slash;
 
   /* Diagnostics follows Posix as much as possible.  */
@@ -6055,7 +6058,7 @@ symlink (char const *filename, char const *linkname)
      directory.  */
   filename_ends_in_slash = IS_DIRECTORY_SEP (filename[strlen (filename) - 1]);
   if (dir_access == 0 || filename_ends_in_slash)
-    flags = SYMBOLIC_LINK_FLAG_DIRECTORY;
+    flags |= SYMBOLIC_LINK_FLAG_DIRECTORY;
 
   tgtfn = (char *)map_w32_filename (filename, NULL);
   if (filename_ends_in_slash)
@@ -6069,6 +6072,16 @@ symlink (char const *filename, char const *linkname)
       if (errno != ENOSYS)
 	{
 	  DWORD w32err = GetLastError ();
+	  /* Remove the flag and try again if the API is not supported */
+	  if (w32err == ERROR_INVALID_PARAMETER)
+	    {
+	      errno = 0;
+	      flags &= ~SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE;
+	      if (create_symbolic_link (linkfn, tgtfn, flags))
+		goto success;
+
+	      w32err = GetLastError ();
+	    }
 
 	  switch (w32err)
 	    {
@@ -6104,6 +6117,8 @@ symlink (char const *filename, char const *linkname)
 	}
       return -1;
     }
+
+ success:
   return 0;
 }
 
-- 
2.37.0.windows.1

